import 'dart:convert';

import 'package:http/http.dart';

import '../phrase_exceptions.dart';
import 'base_phrase_api.dart';

class PlatformPhraseApi extends PhraseApi {
  @override
  Future<PhraseApiResult> getPlatformTranslations({
    required Uri uri,
    required Duration timeout,
  }) async {
    var currentUri = uri;
    String? version;
    String? maxAppVersion;
    String? minAppVersion;
    while (true) {
      final req = Request("GET", currentUri)..followRedirects = false;
      StreamedResponse response = await httpClient.send(req).timeout(timeout);
      int code = response.statusCode;
      if (code == 304) {
        return PhraseApiResult.notModified();
      } else if (code >= 300 && code <= 399) {
        currentUri = Uri.parse(response.headers['location']!);
        version = currentUri.queryParameters['version'];
        maxAppVersion = currentUri.queryParameters['max_version'];
        minAppVersion = currentUri.queryParameters['min_version'];
        continue;
      } else if (code == 404) {
        return PhraseApiResult.notFound();
      } else if (code == 200) {
        if (version == null) throw PhraseApiException("Missing version");
        final arb = await response.stream.bytesToString(utf8);
        return PhraseApiResult.success(
          version,
          arb,
          maxAppVersion: maxAppVersion,
          minAppVersion: minAppVersion,
        );
      } else {
        throw PhraseApiException("Unexpected response code: $code");
      }
    }
  }
}
