import 'dart:html' as html;

import '../phrase_exceptions.dart';
import 'base_phrase_api.dart';

class PlatformPhraseApi extends PhraseApi {
  @override
  Future<PhraseApiResult> getPlatformTranslations({
    required Uri uri,
    required Duration timeout,
  }) async {
    html.HttpRequest? request;
    try {
      request = await html.HttpRequest.request(
        uri.toString(),
        method: 'GET',
      ).timeout(timeout);
    } catch (e) {
      if (e is html.ProgressEvent && e.currentTarget is html.HttpRequest) {
        request = (e).currentTarget as html.HttpRequest;
      }
    }

    final code = request?.status;
    if (code == null) {
      throw PhraseApiException('failed to get updated translations');
    }

    switch (code) {
      case 304:
        return PhraseApiResult.notModified();
      case 404:
        return PhraseApiResult.notFound();
      case 200:
        return _processRequestResponse(request!);
      default:
        throw PhraseApiException("Unexpected response code: $code");
    }
  }

  PhraseApiResult _processRequestResponse(html.HttpRequest request) {
    final currentUri = Uri.parse(request.responseUrl!);
    final version = currentUri.queryParameters['version'];
    if (version == null) throw PhraseApiException("Missing version");
    final arb = request.response;
    return PhraseApiResult.success(version, arb);
  }
}
