import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:phrase/src/cache/phrase_web_cache.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'phrase_web_cache_test.mocks.dart';

@GenerateMocks([
  SharedPreferences,
])
void main() {
  group('PhraseWebCache', () {
    const filename = 'test_file.txt';
    late MockSharedPreferences sharedPreferences;
    late PhraseWebCache cache;

    setUp(() {
      sharedPreferences = MockSharedPreferences();
      cache = PhraseWebCache(
        getSharedPreferencesCallback: () async => sharedPreferences,
      );

      when(sharedPreferences.setString(any, any)).thenAnswer((_) async => true);
      when(sharedPreferences.remove(any)).thenAnswer((_) async => true);
    });

    test('write', () async {
      const contents = 'test contents';
      await cache.write(filename, contents);

      verify(sharedPreferences.setString(filename, contents));
      verifyNoMoreInteractions(sharedPreferences);
    });

    test('read', () async {
      const contents = 'test contents';
      when(sharedPreferences.getString(filename)).thenAnswer((_) => contents);

      final readContents = await cache.read(filename);
      expect(readContents, equals(contents));

      verify(sharedPreferences.getString(filename));
      verifyNoMoreInteractions(sharedPreferences);
    });

    test('delete', () async {
      await cache.delete(filename);

      verify(sharedPreferences.remove(filename));
      verifyNoMoreInteractions(sharedPreferences);
    });
  });
}
